from gpanel import *
from sys import exit

neighbours = {
    'Althaus':('Bellevue', 'Dom', 'Enge', 'Friedhof'), 
    'Bellevue':('Althaus', 'City', 'Dom'), 
    'City':('Bellevue', 'Dom', 'Friedhof'), 
    'Dom':('Althaus', 'Bellevue', 'City', 'Enge', 'Friedhof'), 
    'Enge':('Althaus', 'Dom'), 
    'Friedhof':('Althaus', 'City', 'Dom')}

distances = {  # keys are ordered alphabetically
   ('Althaus', 'Bellevue'):5, ('Althaus', 'Dom'):3, 
   ('Althaus', 'Enge'):6, ('Althaus', 'Friedhof'):15,
   ('Bellevue', 'City'):3, ('Bellevue', 'Dom'):13, 
   ('City', 'Dom'):4, ('City', 'Friedhof'):3, 
   ('Dom', 'Enge'):2, ('Dom', 'Friedhof'):12}

locations = {
    'Althaus':(5, 9), 
    'Bellevue':(1, 7), 
    'City':(3, 3), 
    'Dom':(9, 5), 
    'Enge':(11, 9), 
    'Friedhof':(7, 1)}

stations = {
    # temporary or permanent, 
    # distance to start station (1000 if infinite), 
    # previous station (emtpy if not yet set)
    'Althaus':['temporary', 1000, ''],
    'Bellevue':['temporary', 1000, ''],
    'City':['temporary', 1000, ''],
    'Dom':['temporary', 1000, ''],
    'Enge':['temporary', 1000, ''],
    'Friedhof':['temporary', 1000, '']}

def getNeighbourDistance(station1, station2):
    if station1 < station2:  # compare alphabetically
        return distances[(station1, station2)]
    return distances[(station2, station1)]

def drawGraph():
    image("sprites/city.png", 0, 0)
    lineWidth(2)
    setColor("black")
    for station in locations:
        move(locations[station])  
        circle(0.4) # draw station as circle
        for neighbour in neighbours[station]:
            drawConnection(station, neighbour) # draw connecting lines

    # fill with color to indicate temporary or permanent        
    for station in locations:
        if stations[station][0] == "temporary":
            setColor("yellow")
        elif stations[station][0] == "permanent":
            setColor("green")
        pt = locations[station]    
        move(pt)
        fillCircle(0.38) 

        # show station indentifier
        setColor("black")
        text(pt[0] - 0.15, pt[1] - 0.15, station)
        
        # show distance to start station and previous station
        dist = stations[station][1]
        dist = "oo" if dist == 1000 else str(dist)  # 1000->infinite sign
        text(pt[0] - 0.5, pt[1] + 0.5, 
            dist + "(" + stations[station][2] + ")")
    repaint()
    
def drawConnection(start, end):
    ptStart = locations[start]
    ptEnd = locations[end]
    line(ptStart, ptEnd)  
    distance = getNeighbourDistance(start, end)
    text(getDividingPoint(ptStart, ptEnd, 0.5), str(distance))
      
def getTemporaryNeighbours(station):
    li = []
    for n in neighbours[station]:
        if stations[n][0] == "temporary":
            li.append(n)
    return li

def getNearestTemporaryStation():
    minstation = None
    min = 1000
    for n in stations:
        if stations[n][0] == "temporary" and stations[n][1] <= min:
            min = stations[n][1]
            minstation = n
    return minstation

def updateStations(station):
    stations[station][0] = "permanent"  # make station permanent
    currentDist = stations[station][1]                  
    for n in getTemporaryNeighbours(station):
        newDist = currentDist + getNeighbourDistance(station, n)
        dist = stations[n][1]
        if newDist < dist:
            stations[n][1] = newDist
            stations[n][2]= station

makeGPanel(Size(700, 500))
enableRepaint(False)
window(0, 14, 0, 10)
title("Shortest Path (Dijkstra Algorithm)")
font(Font("Arial", Font.PLAIN, 20))
addStatusBar(30)
drawGraph()
startStation = ""
setStatusText("Select start station")
while not startStation in locations:
    startStation = inputString("Start station", False)
    if startStation == None:
        dispose()
        exit()

stations[startStation][1] = 0    
drawGraph()
steps = 0
setStatusText("Press any key to step through algorithm...")

# Perform Dijkstra algorithm
while True:
   if getKeyWait():
        currentStation = getNearestTemporaryStation()
        if currentStation == None:
            break 
        updateStations(currentStation)
        drawGraph()
        steps += 1
        setStatusText("Step #" + str(steps) + " - continue...")

setStatusText("Algorithm finished.")
targetStation = ""
while not targetStation in locations:
    targetStation = inputString("Target station", False)
    if targetStation == None:
        dispose()
        exit()

# Build track                
station = targetStation
path = [station]
while station != startStation:
    station = stations[station][2]
    path.append(station)
track = path[::-1] # reverse    
lineWidth(4)
setColor("red")
trackLength = 0
for i in range(len(path) - 1):
    n1 = track[i]
    n2 = track[i + 1]
    trackLength += getNeighbourDistance(n1, n2)
    line(locations[n1], locations[n2])  
setStatusText("Shortest route from " + startStation 
    + " to " + targetStation  + ": " + str(track) 
    + " Length = " + str(trackLength))
repaint()    
    